/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.tools.jmx.api;

import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.CONF_CONTAINER_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.CONF_JMX_JNDI_PORT;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.CONF_JMX_RMI_PORT;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.PETALS14_CONF_CONTAINER_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.PETALS14_CONF_JMX_JNDI_PORT;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.ContainerKeyNames.PETALS14_CONF_JMX_RMI_PORT;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.DomainKeyNames.CONF_DOMAIN_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.DomainKeyNames.PETALS14_CONF_DOMAIN_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.MBean.PETALSADMIN_OBJECT_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.PETALS14_RETRIEVE_CONTAINERS_CONFIGURATION;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.RETRIEVE_CONTAINERS_CONFIGURATION;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.SHUTDOWN;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.STOP;

import java.io.IOException;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminServiceErrorException;

/**
 * Client API of the JMX Petals Admin service.
 * 
 * @author rnaudin - EBM Websourcing
 * 
 */
public class PetalsAdminServiceClient extends PetalsAbstractServiceClient implements
        IPetalsAdminService {
    
    private final boolean isPetalsServer20Older;

    /**
     * Creates a new instance of {@link PetalsAdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @param isPetalsServer20Older
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     */
    public PetalsAdminServiceClient(final String petalsDomain,
            final MBeanServerConnection beanServerConnection,
            final boolean isPetalsServer20Older)
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException {
        super(petalsDomain, beanServerConnection);
        
        this.isPetalsServer20Older = isPetalsServer20Older;

        try {
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(
                    PETALSADMIN_OBJECT_NAME, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new PetalsAdminDoesNotExistException();
            }
        } catch (IOException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * Stop the PEtALS container.
     * 
     * @see PetalsAdminServiceInterface#stopContainer()
     * @throws PerformActionErrorException
     */
    public String stopContainer() throws PerformActionErrorException {

        final Object[] objectsLoad = new Object[0];
        final String[] stringsLoad = new String[0];

        return (String) this.performAction(STOP, objectsLoad, stringsLoad);
    }

    /**
     * Shut down the PEtALS container.
     * 
     * @see PetalsAdminServiceInterface#shutdownContainer()
     * @throws PerformActionErrorException
     */
    public String shutdownContainer() throws PerformActionErrorException {

        final Object[] objectsLoad = new Object[0];
        final String[] stringsLoad = new String[0];

        return (String) this.performAction(SHUTDOWN, objectsLoad, stringsLoad);
    }

    /**
     * Retrieve the configuration of PEtALS containers.
     * 
     * @see PetalsAdminServiceInterface#retrieveContainersConfiguration()
     * @throws PetalsAdminServiceErrorException
     */
    public Set<Map<String, String>> retrieveContainersConfiguration()
            throws PetalsAdminServiceErrorException {

        try {
            Object confs;
            if (this.isPetalsServer20Older) {
                confs = this.performAction(PETALS14_RETRIEVE_CONTAINERS_CONFIGURATION, new Object[0], new String[0]);                
            }
            else {
                confs = this.performAction(RETRIEVE_CONTAINERS_CONFIGURATION, new Object[0], new String[0]);                
            }
            
            final Set<Map<String, String>> result;
            if (confs == null) {
                result = new HashSet<Map<String, String>>();
            } else if (confs instanceof Set) {
                result = (Set<Map<String, String>>) confs;
            } else {
                throw new PetalsAdminServiceErrorException("Unexpected result type");
            }

            return result;
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * Retrieve the configuration property of a PEtALS container configuration.
     * The property name is adjusted according to the PEtALS container version.
     * 
     * @param containerConfiguration
     *            The configuration of a container. This configuration has been
     *            returned previously by
     *            {@link PetalsAdminServiceClient#retrieveContainersConfiguration()}.
     * @param propertyName
     *            The property name of the expected value. All properties are
     *            defined in {@link IPetalsAdminService.ContainerKeyNames} and
     *            {@link IPetalsAdminService.DomainKeyNames}. The property name
     *            does not have to be prefixed by PETALS14.
     */
    public String getContainerProperty(final Map<String, String> containerConfiguration,
            final String propertyName) {
        String adjustedPropertyName = propertyName;
        if (this.isPetalsServer20Older) {
            if (CONF_DOMAIN_NAME.equals(propertyName)) {
                adjustedPropertyName = PETALS14_CONF_DOMAIN_NAME;
            } else if (CONF_CONTAINER_NAME.equals(propertyName)) {
                adjustedPropertyName = PETALS14_CONF_CONTAINER_NAME;
            } else if (CONF_JMX_RMI_PORT.equals(propertyName)) {
                adjustedPropertyName = PETALS14_CONF_JMX_RMI_PORT;
            } else if (CONF_JMX_JNDI_PORT.equals(propertyName)) {
                adjustedPropertyName = PETALS14_CONF_JMX_JNDI_PORT;
            }
        }
        return containerConfiguration.get(adjustedPropertyName);
    }
}
